%% Run spindle model applying a ramp-and-hold stretch
% This scripts runs the updated Maltenfort and Burke 2003 muscle spindle
% model and prodcues the figure that corresponds to Figure 1 of the
% physiome article

% muscle_spindle_maltenfort_and_burke_2003.m must be in same folder or add
% path to run the simulation

% set true if digitized data from original paper shall be plotted
% (Maltenfort_and_Burke_2003_Fig4A_digitized.csv and
% Maltenfort_and_Burke_2003_Fig4B_digitized.csv must be in same folder or
% to add path)
plot_paper_data = true; 

%% Simulation settings

% Two 6mm ramp and hold stretches are applied: 
% a: ramp velocity 5 mm/s
% b: ramp velocity 30 mm/s

% Simulation times in seconds
dt = 1e-3; % 
t_end_a = 2.5; % 
t_end_b = 0.4; % 
times_a = 0:dt:t_end_a;
times_b = 0:dt:t_end_b;
ndt_a = length(times_a);
ndt_b = length(times_b);

number_of_simulations = 3; % 3 for each figure

% Define displacement
disp_max = 6; % [mm] 
disp_min = 0; % [mm] 
disp_start_a = 200e-3; % [s] (Figure A)
disp_start_b = 50e-3; % [s] (Figure B)
disp_velo_a = 5; % [mm/s] (Figure A)
disp_velo_b = 30; % [mm/s] Figure B

% Create displacements
idx_start_a = disp_start_a/dt+1;
idx_end_a = disp_start_a/dt+disp_max/disp_velo_a/dt+1;
displacements_a(idx_start_a:idx_end_a) = linspace(disp_min,disp_max,idx_end_a-idx_start_a+1); % ramp
displacements_a(idx_end_a+1:ndt_a) = disp_max; % hold

idx_start_b = disp_start_b/dt+1;
idx_end_b = disp_start_b/dt+disp_max/disp_velo_b/dt+1;
displacements_b(idx_start_b:idx_end_b) = linspace(disp_min,disp_max,idx_end_b-idx_start_b+1); % ramp
displacements_b(idx_end_b+1:ndt_b) = disp_max; % hold

% gamma drive in pps
gamma_s = [0,50,0]; 
gamma_d = [0,0,50];

%% Run the model

% Figure A (ramp velocity 5 mm/s)

% Preallocate output vector
Ia_a = zeros(ndt_a,number_of_simulations);

for i = 1:number_of_simulations
      
    % Run the spindle model
    Ia_a(:,i) = muscle_spindle_maltenfort_and_burke_2003(...
        times_a,displacements_a,gamma_s(i).*ones(1,ndt_a),gamma_d(i).*ones(1,ndt_a),dt.*1e3); % dt must be in ms for model script
    
end


% Figure B (ramp velocity30 mm/s)

% Preallocate output vector
Ia_b = zeros(ndt_b,number_of_simulations);

for i = 1:number_of_simulations
      
    % Run the spindle model
    Ia_b(:,i) = muscle_spindle_maltenfort_and_burke_2003(...
        times_b,displacements_b,gamma_s(i).*ones(1,ndt_b),gamma_d(i).*ones(1,ndt_b),dt.*1e3); % dt must be in ms for model script
    
end

%% Create plot

if plot_paper_data
    MaltenfortandBurke2003Fig4Adigitized = readtable('Maltenfort_and_Burke_2003_Fig4A_digitized.csv');
    MaltenfortandBurke2003Fig4Bdigitized = readtable('Maltenfort_and_Burke_2003_Fig4B_digitized.csv');
end

line_colors = [0 0 1;1 0.5 0;0 0.8 0];
fig1=figure;
set(fig1,'defaultAxesColorOrder',[0 0 0; 0.5 0.5 0.5]);

% Figure A
subplot(1,2,1),

% displacement
yyaxis right
plot(times_a,displacements_a,'HandleVisibility','off'),
ylabel('Displacement (mm)'),ylim([0,6.5])

% matlab simulation results
yyaxis left
for i = 1:3
    plot(times_a,Ia_a(:,i),'-', 'Color',line_colors(i,:),'LineWidth',1), hold on,
end

% digitized data from publication 
if plot_paper_data
    Figure4A_data = table2array(MaltenfortandBurke2003Fig4Adigitized);
    for i = 1:3
        plot(Figure4A_data(:,1),Figure4A_data(:,i+1),'o','MarkerSize',4,'Color',line_colors(i,:)), hold on,
    end
end
    
% labels
legend({'\gamma_s = \gamma_d = 0 pps (matlab)', '\gamma_s = 50 pps (matlab)', '\gamma_d = 50 pps (matlab)',...
    '\gamma_s = \gamma_d = 0 pps (data)', '\gamma_s = 50 pps (data)', '\gamma_d = 50 pps (data)'},...
    'Location','EastOutside'),
xlabel('Time (s)'),ylabel('Ia firing rate (pps)'),
title('A: ramp velocity 5 mm/s'),
xlim([0,times_a(end)]),ylim([0,150])


% Figure B
subplot(1,2,2),

% displacement
yyaxis right
plot(times_b,displacements_b,'HandleVisibility','off'),
ylabel('Displacement (mm)'),ylim([0,6.5]),

% matlab simulation results
yyaxis left
for i = 1:3
    plot(times_b,Ia_b(:,i),'-','Color',line_colors(i,:),'LineWidth',1), hold on,
end

% digitized data from publication 
if plot_paper_data
    Figure4B_data = table2array(MaltenfortandBurke2003Fig4Bdigitized);
    for i = 1:3
        plot(Figure4B_data(:,1),Figure4B_data(:,i+1),'o','MarkerSize',4,'Color',line_colors(i,:)), hold on,
    end
end

% further labels
legend({'\gamma_s = \gamma_d = 0 pps (matlab)', '\gamma_s = 50 pps (matlab)', '\gamma_d = 50 pps (matlab)',...
    '\gamma_s = \gamma_d = 0 pps (data)', '\gamma_s = 50 pps (data)', '\gamma_d = 50 pps (data)'},...
    'Location','EastOutside'),
xlabel('Time (s)'),ylabel('Ia firing rate (pps)')
title('B: ramp velocity 30 mm/s'),
xlim([0,times_b(end)]),ylim([0,200])
