%% Run spindle model applying a sinusoidal stretch
% This scripts runs the updated Maltenfort and Burke 2003 muscle spindle
% model and prodcues the figure that corresponds to Figure 2 of the
% physiome article

% muscle_spindle_maltenfort_and_burke_2003.m must be in same folder or add
% path to run the simulation

% set true if digitized data from original paper shall be plotted
% (Maltenfort_and_Burke_2003_Fig5A_digitized.csv, *5B*, *5C*, *5D* must be
% in same folder or add path)
plot_paper_data = true; 

%% Simulation settings

% Two periods of a sinusoidal stretch are applied under varying fusimotor
% drive

% Simulation time
dt = 1e-3; % [s]
t_end = 2; % [s]
times = 0:dt:t_end;
ndt = length(times);

number_of_simulations = 16;

% Define displacement
disp_max = 0.7; % amplitude [mm]
disp_min = 4; % mean [mm]
disp_velo = 1; % [Hz]

% Create displacements
displacements = disp_max*(cos(2*pi*disp_velo*(0:dt:(ndt-1)*dt)))+disp_min; % two periods

% gamma drive in pps for 16 simulations
gamma_s = [0,50,75,125,... % Figure 5A
    zeros(1,4),... % Figure 5B
    0,50,75,125,... % Figure 5C
    70.*ones(1,4)]; % Figure 5D
gamma_d = [zeros(1,4),... % Figure 5A
    0,50,75,125,... % Figure 5B
    125.*ones(1,4),... 5 Figure 5C
    0,50,75,125]; %Figure 5D 

%% Run the model

% Preallocate output vector
Ia = zeros(ndt,number_of_simulations);

for i = 1:number_of_simulations
      
    % Run the spindle model
    Ia(:,i) = muscle_spindle_maltenfort_and_burke_2003(...
        times,displacements,gamma_s(i).*ones(1,ndt),gamma_d(i).*ones(1,ndt),dt.*1e3); % dt must be in ms for model script
    
end

%% Create plot

if plot_paper_data
    MaltenfortandBurke2003Fig5Adigitized = readtable('Maltenfort_and_Burke_2003_Fig5A_digitized.csv');
    MaltenfortandBurke2003Fig5Bdigitized = readtable('Maltenfort_and_Burke_2003_Fig5B_digitized.csv');
    MaltenfortandBurke2003Fig5Cdigitized = readtable('Maltenfort_and_Burke_2003_Fig5C_digitized.csv');
    MaltenfortandBurke2003Fig5Ddigitized = readtable('Maltenfort_and_Burke_2003_Fig5D_digitized.csv');
end

line_colors = [0 0 1;1 0.5 0;0 0.8 0;0 0 0];
fig2=figure;
set(fig2,'defaultAxesColorOrder',[0 0 0; 0.5 0.5 0.5]);

% The second of two sinusoidals is plotted with respect to the
% phase
phase = linspace(0,360,(ndt-1)/2+1);

% Figure A
subplot(2,2,1),

% displacements 
yyaxis right
plot(phase,displacements((ndt-1)/2+1:end),'HandleVisibility','off'),
ylabel('Displacement (mm)'),ylim([3.3,4.7])

% matlab simulation results
yyaxis left
for i = 1:4
    plot(phase,Ia((ndt-1)/2+1:end,i),'-','Color',line_colors(i,:),'LineWidth',1), hold on,
end

% digitized data from publication 
if plot_paper_data
    Figure5A_data = table2array(MaltenfortandBurke2003Fig5Adigitized);
    for i = 1:4
        plot(Figure5A_data(:,1),Figure5A_data(:,i+1),'o','MarkerSize',4,'Color',line_colors(i,:)), hold on,
    end
end

% Labels
xlabel('Phase of displacement (deg)'),ylabel('Ia firing rate (pps)')
legend({'0 pps (matlab)', '50 pps (matlab)', '75 pps (matlab)', '125 pps (matlab)',...
    '0 pps (data)', '50 pps (data)', '75 pps (data)', '125 pps (data)'},...
    'Location','EastOutside'),
title('A: \gamma_s alone'),
xlim([0,360]),ylim([0,300])

% Figure B
subplot(2,2,2),

% displacements 
yyaxis right
plot(phase,displacements((ndt-1)/2+1:end),'HandleVisibility','off'),
ylabel('Displacement (mm)'),ylim([3.3,4.7])

% matlab simulation results
yyaxis left
for i = 1:4
    plot(phase,Ia((ndt-1)/2+1:end,4+i),'-','Color',line_colors(i,:),'LineWidth',1), hold on,
end

% digitized data from publication 
if plot_paper_data
    Figure5B_data = table2array(MaltenfortandBurke2003Fig5Bdigitized);
    for i = 1:4
        plot(Figure5B_data(:,1),Figure5B_data(:,i+1),'o','MarkerSize',4,'Color',line_colors(i,:)), hold on,
    end
end

% Labels
xlabel('Phase of displacement (deg)'),ylabel('Ia firing rate (pps)')
legend({'0 pps (matlab)', '50 pps (matlab)', '75 pps (matlab)', '125 pps (matlab)',...
    '0 pps (data)', '50 pps (data)', '75 pps (data)', '125 pps (data)'},...
    'Location','EastOutside'),
title('B: \gamma_d alone'),
xlim([0,360]),ylim([0,300])


% Figure C
subplot(2,2,3),

% displacements 
yyaxis right
plot(phase,displacements((ndt-1)/2+1:end),'HandleVisibility','off'),
ylabel('Displacement (mm)'),ylim([3.3,4.7])

% matlab simulation results 
yyaxis left
for i = 1:4
    plot(phase,Ia((ndt-1)/2+1:end,8+i),'-','Color',line_colors(i,:),'LineWidth',1), hold on,
end

% digitized data from publication 
if plot_paper_data
    Figure5C_data = table2array(MaltenfortandBurke2003Fig5Cdigitized);
    for i = 1:4
        plot(Figure5C_data(:,1),Figure5C_data(:,i+1),'o','MarkerSize',4,'Color',line_colors(i,:)), hold on,
    end
end

% labels
xlabel('Phase of displacement (deg)'),ylabel('Ia firing rate (pps)')
legend({'0 pps (matlab)', '50 pps (matlab)', '75 pps (matlab)', '125 pps (matlab)',...
    '0 pps (data)', '50 pps (data)', '75 pps (data)', '125 pps (data)'},...
    'Location','EastOutside'),
title('C: \gamma_s + 125 pps \gamma_d'),
xlim([0,360]),ylim([0,300])       


% Figure D
subplot(2,2,4),

% displacements 
yyaxis right
plot(phase,displacements((ndt-1)/2+1:end),'HandleVisibility','off'),
ylabel('Displacement (mm)'),ylim([3.3,4.7])

% matlab simulation results
yyaxis left
for i = 1:4
    plot(phase,Ia((ndt-1)/2+1:end,12+i),'-','Color',line_colors(i,:),'LineWidth',1), hold on,
end

% digitized data from publication 
if plot_paper_data
    Figure5D_data = table2array(MaltenfortandBurke2003Fig5Ddigitized);
    for i = 1:4
        plot(Figure5D_data(:,1),Figure5D_data(:,i+1),'o','MarkerSize',4,'Color',line_colors(i,:)), hold on,
    end
end

% labels
xlabel('Phase of displacement (deg)'),ylabel('Ia firing rate (pps)')
legend({'0 pps (matlab)', '50 pps (matlab)', '75 pps (matlab)', '125 pps (matlab)',...
    '0 pps (data)', '50 pps (data)', '75 pps (data)', '125 pps (data)'},...
    'Location','EastOutside'),
title('D: \gamma_d + 70 pps \gamma_s'),
xlim([0,360]),ylim([0,300])

