%% Compare MATLAB and cellml versions of the spindle model
% This scripts runs the updated Maltenfort and Burke 2003 muscle spindle
% model, loads data from cellml simulations and produces the figure that
% corresponds to Figure 3 of the physiome article.
% The stretch scenario is a ramp-and-hold tsretch followed by a wait period
% and a sinusoidal stretch.

% muscle_spindle_maltenfort_and_burke_2003.m must be in same folder or add
% path to run the simulation

% set true if data from cellml simulations shall be plotted (file Fig03.csv
% must be placed in same folder or add path)
plot_cellml_data = true; 

%% Matlab simulation settings

% Simulation time
dt = 1e-3; % [s]
t_end = 3; % [s]
times = 0:dt:t_end;
ndt = length(times);

% Create displacements
displacements = zeros(1,ndt);
displacements(51:251) = linspace(0,6,201); % ramp to 6 mm with 30 mm/s
displacements(252:401) = 6; % hold for 150 ms
displacements(402:1000) = 0; % wait
displacements(1001:end) = 0.7*(cos(2*pi*(0:dt:(2000)*dt)))+4; % two periods

% Gamma drive
gamma_s = zeros(1,ndt);
gamma_s(1001:end) = 50; % during sinusoidal stretch

gamma_d = zeros(1,ndt);
gamma_d(1001:end) = 125; 


%% Run the MATLAB model
      
% Run the spindle model
Ia = muscle_spindle_maltenfort_and_burke_2003(...
    times,displacements,gamma_s,gamma_d,dt.*1e3); % dt must be in ms for model script
    
%% Create plot

% Import results from cellml simulations
if plot_cellml_data   
data = readtable('Fig03.csv');
times_cellml = data.main_Time_millisecond_.*1e-3; % convert from ms to s
Ia_cellml = data.main_Ia_pulses_per_second_;
phase_cellml = linspace(0,360,10001); % phase of sinusoidal cycle
end

line_colors = [0 0 1;1 0.5 0];
fig3=figure;

% The second of two sinusoidals is plotted with respect to the
% phase
phase = linspace(0,360,1001);

% Figure A
subplot(1,2,1),

% matlab simulation results ramp-and-hold
plot(times(1:401),Ia(1:401),'-','Color',line_colors(1,:),'LineWidth',1), hold on,

% cellml simulation results
if plot_cellml_data
    plot(times_cellml(1:4001),Ia_cellml(1:4001),'-','Color',line_colors(2,:),'LineWidth',1)
end

% Labels
xlabel('Time (s)'),ylabel('Ia firing rate (pps)')
legend({'MATLAB simulation results', 'CellML simulation results'}),
title('ramp-and-hold stretch: \gamma_s = 0 pps, \gamma_d = 0 pps'),
xlim([0,0.4]),ylim([0,150])

% Figure B
subplot(1,2,2),

% matlab simulation results second of two sinusoidal stretch cycles
plot(phase,Ia(2001:end),'-','Color',line_colors(1,:),'LineWidth',1), hold on,

% digitized data from publication 
if plot_cellml_data
    plot(phase_cellml,Ia_cellml(20001:end),'-','Color',line_colors(2,:),'LineWidth',1)
end

% Labels
xlabel('Phase (deg)'),ylabel('Ia firing rate (pps)')
legend({'MATLAB simulation results', 'CellML simulation results'}),
title('sinusoidal stretch: \gamma_s = 50 pps, \gamma_d = 125 pps'),
xlim([0,360]),ylim([0,300])
