function [Ia] = muscle_spindle_maltenfort_and_burke_2003(times,displacements,gamma_static,gamma_dynamic,dt)
%% Implementation of the UPDATED muscle spindle model of Maltenfort and Burke (2003)
%
% This code contains the corrections with respect to the published
% equations which are necessary to reproduce the published results (see
% accompanying Physiome article)
% Original publication: Maltenfort & Burke 2003, Spindle Model Responsive
% to Mixed Fusimotor Inputs and Testable Predictions of Beta Feedback Effects,
% J Neurophysiol.
%
%   Out: Ia is the overall afferent spindle feedback in pps. 
%   In: times is a vector containing all time points the model is evaluated
%       at in milliseconds.
%   In: displacements is a vector containing the muscle fiber displacements
%       in millimeters.
%   In: gamma_static is vector containing the static fusimotor drive
%       in pps.
%   In: gamma_dynamic is vector containing the dynamic fusimotor drive
%       in pps.
%   In: dt is the time step size in milliseconds.

%%

num_of_dts = length(times);

% Check for correct dimensions 
if(num_of_dts == length(displacements) && num_of_dts == length(displacements) && ...
        num_of_dts == length(gamma_static) && num_of_dts == length(gamma_dynamic))
   % Do nothing - Correct Input  
else
   disp('Error: Input Vectors must have the same length.')   
end

% Preallocate the output vector
Ia = zeros(num_of_dts,1);
% Get spindle parameters
p = get_parameters;
% Initial values for static and dynamic feedback
Delta_Rs = 0;
Delta_Rd = 0;

%% Filtering of displacements and velocites

% For passive and static fusimotor drive
tau_ps = 20; %[ms]
x_ps = zeros(1,num_of_dts); % preallocation
x_ps(1) = displacements(1);
velocities_ps = zeros(1,num_of_dts);
for time=2:num_of_dts
    x_ps(time) = exp(-dt/tau_ps)*x_ps(time-1)+(1-exp(-dt/tau_ps))*displacements(time);
    velocities_ps(time) = (displacements(time)-x_ps(time))/tau_ps;
end

% For dynamic fusimotor drive
tau_d = 100; %[ms]
x_d = zeros(1,num_of_dts); % preallocation
x_d(1) = displacements(1);
velocities_d = zeros(1,num_of_dts);
for time=2:num_of_dts
    x_d(time) = exp(-dt/tau_d)*x_d(time-1)+(1-exp(-dt/tau_d))*displacements(time);
    velocities_d(time) = (displacements(time)-x_d(time))/tau_d;
end

% Convert velocities into mm/s
velocities_ps = velocities_ps.*1000;
velocities_d = velocities_d.*1000;

% Solve the model
for time=1:num_of_dts    
    %% Evaluate passive part of the muscle spindle feedback
    S_ss = p(1);
    if(velocities_ps(time) > 0)
       Q = p(2)*velocities_ps(time)^p(4); 
       S_v = p(5)*velocities_ps(time)^p(6);
    else
       Q = p(3)*abs(velocities_ps(time))^p(4);
       S_v = 0;
    end
    R_passive = (S_v + S_ss)*displacements(time) + Q;
    
    %% Evaluate active (static) part of the muscle spindle feedback
    if(gamma_static(time) > 0)
        B = p(7)*gamma_static(time); 
        S_ss = p(8)*(1-exp(-(gamma_static(time)/p(9))^p(10)));
        if(velocities_ps(time) > 0)
           Q = p(11)*(gamma_static(time)/(p(12)+gamma_static(time)))*velocities_ps(time)^p(4); 
           g = 0;
        else
           Q = abs(velocities_ps(time))^p(4)*(p(16)+p(17)*(gamma_static(time)/(p(12)+gamma_static(time))));
           g = p(15)*abs(velocities_ps(time))^p(6);
        end
        S_v = p(13)*abs(velocities_ps(time))*gamma_static(time)/(p(14)+gamma_static(time))+g;
        Delta_Rs = (S_v + S_ss)*displacements(time) + Q + B;
    end
    
    %% Evaluate active (dynamic) part of the muscle spindle feedback
    if(gamma_dynamic(time) > 0)
        B = p(18)*gamma_dynamic(time);
        S_ss = p(19)*(1-exp(-(gamma_dynamic(time)/p(20))^p(21)));
        if(velocities_d(time) > 0)
            a = p(22)*(1-exp(-(gamma_dynamic(time)/p(23))^p(24))); % modified with respect to original publication
            c = p(25)*(1-exp(-(gamma_dynamic(time)/p(26))^p(27))); % modified with respect to original publication
        else
            a = 0;
            c = 0;
        end
        b = (gamma_dynamic(time)/p(28)+p(29))^2;
        d = p(30)/(1+gamma_dynamic(time)/p(31)); % modified with respect to original publication
        Q = a*atan(velocities_d(time)/b);
        S_v = c*atan(velocities_d(time)/d);
        Delta_Rd = (S_v + S_ss)*displacements(time) + Q + B;
    end
    
    %% Occlusion
    Ia(time) = R_passive + Delta_Rs + Delta_Rd; 
    if(Delta_Rs > 0) && (Delta_Rd > 0)
        Ia(time) = Ia(time) - 1/(1/Delta_Rs + 1/Delta_Rd);
    end
    % Exclude negative firing rates
    Ia(Ia<0)=0;
    
end 

end

% Model parameters
function [p] = get_parameters()
    p(1) = 3.9; 
    p(2) = 2.9;
    p(3) = -5.9;
    p(4) = 0.88;
    p(5) = 2.5;
    p(6) = 0.04;
    p(7) = 0.74;
    p(8) = 2.14;
    p(9) = 78.74;
    p(10) = 2.72;
    p(11) = -2.7;
    p(12) = 62;
    p(13) = 0.16;
    p(14) = 143;
    p(15) = 2.5;
    p(16) = 2.9;
    p(17) = 2.7;
    p(18) = 0.55;
    p(19) = -1.8;
    p(20) = 88.06;
    p(21) = 2.32;
    p(22) = 101;
    p(23) = 168;
    p(24) = 1.94;
    p(25) = 64.4;
    p(26) = 225;
    p(27) = 1.56;
    p(28) = 116;
    p(29) = 0.05;
    p(30) = 8.6;
    p(31) = 39;
end

